import json
from io import StringIO

from consents.tests.tests_consent_operations import BaseTestCase
from providers.models import Provider


class TestProviderSchemaCreation(BaseTestCase):
    def test_create_provider_schema_fails(self):
        """
        Ensure provider schema with duplicate group ids fails.
        """
        provider_schema = {
            "id": "h5p-0",
            "name": "H5P",
            "description": "Open-source content collaboration framework",
            "groups": [
                {
                    "id": "default_group",
                    "label": "Default group",
                    "description": "default",
                    "showVerbDetails": True,
                    "purposeOfCollection": "Lorem Ipsum",
                    "verbs": [
                        {
                            "id": "https://xapi.elearn.rwth-aachen.de/definitions/lms/verbs/unlocked",
                            "label": "Unlocked",
                            "description": "Actor unlocked an object",
                            "defaultConsent": True,
                            "objects": [
                                {
                                    "id": "https://xapi.elearn.rwth-aachen.de/definitions/lms/activities/my-random-object-id",
                                    "label": "Course",
                                    "defaultConsent": True,
                                    "matching": "id",
                                    "definition": {
                                        "type": "https://xapi.elearn.rwth-aachen.de/definitions/lms/activities/course",
                                        "name": {
                                            "enUS": "A course within an LMS. Contains learning materials and activities"
                                        },
                                    },
                                },
                            ],
                        }
                    ],
                    "isDefault": True,
                },
                {
                    "id": "default_group",
                    "label": "Group 2",
                    "description": "Lorem ipsum",
                    "showVerbDetails": True,
                    "purposeOfCollection": "Lorem Ipsum",
                    "verbs": [
                        {
                            "id": "http://h5p.example.com/expapi/verbs/interacted",
                            "label": "Interacted",
                            "description": "Lorem ipsum",
                            "defaultConsent": True,
                            "objects": [
                                {
                                    "id": "http://h5p.example.com/expapi/activity/ofN2ODcnLRaVu30lUpzrWPqF2AcG7g46",
                                    "label": "1.2.3 Kappenventil",
                                    "defaultConsent": True,
                                    "matching": "definitionType",
                                    "definition": {
                                        "type": "http://h5p.example.com/expapi/activity/ofN2ODcnLRaVu30lUpzrWPqF2AcG7g46",
                                        "name": {"enUS": "1.2.3 Kappenventil"},
                                    },
                                }
                            ],
                        },
                        {
                            "id": "http://h5p.example.com/expapi/verbs/answered",
                            "label": "Answered",
                            "description": "lorem ipsum",
                            "defaultConsent": False,
                            "objects": [
                                {
                                    "id": "http://h5p.example.com/expapi/activity/K34IszYvGE4R0cC72Ean6msLfLCJtQ8b",
                                    "label": "7.2.1 Ventil Basics",
                                    "defaultConsent": False,
                                    "matching": "definitionType",
                                    "definition": {
                                        "type": "http://h5p.example.com/expapi/activity/K34IszYvGE4R0cC72Ean6msLfLCJtQ8b",
                                        "name": {"enUS": "7.2.1 Ventil Basics"},
                                    },
                                }
                            ],
                        },
                    ],
                    "isDefault": False,
                },
            ],
            "essentialVerbs": [],
        }

        # Upload provider schema
        with StringIO(json.dumps(provider_schema)) as fp:
            response = self.provider_client.put(
                "/api/v1/consents/provider/create",
                {"provider-schema": fp},
                format="multipart",
            )
            self.assertEqual(response.status_code, 400)
            self.assertEqual(
                response.json()["message"], "Provider schema contains ambiguous group ids."
            )
