import { Component, Inject, Input } from '@angular/core';
import {MAT_DIALOG_DATA, MatDialogRef, MatDialogModule} from '@angular/material/dialog';
import {NgFor, NgIf} from '@angular/common';
import {MatButtonModule} from '@angular/material/button';
import {FormsModule} from '@angular/forms';
import {MatInputModule} from '@angular/material/input';
import {MatFormFieldModule} from '@angular/material/form-field';
import { AnalyticsTokenConsent, XApiVerbConsentedWithProvider } from '../../analytics-engine.component';
import { MatTooltipModule } from '@angular/material/tooltip';
import { MatListModule } from '@angular/material/list';
import { MatIconModule } from '@angular/material/icon';
import { AnalyticsTokenVerb, ApiService, Provider, ProviderAnalyticsToken } from 'src/app/services/api.service';
import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'verbs-modal',
  templateUrl: 'verbs-modal.component.html',
  standalone: true,
  styleUrls: ['verbs-modal.component.scss'],
  imports:[
    MatFormFieldModule,
    MatInputModule,
    FormsModule,
    MatButtonModule,
    NgIf,
    MatDialogModule,
    MatTooltipModule,
    MatListModule,
    NgFor,
    MatIconModule
    ],})
export class VerbsModal {

  providers : Provider[];
  analysis : AnalyticsTokenConsent;
  providerMapping : {
    provider : Provider, 
    notConsentedVerbs : XApiVerbConsentedWithProvider[], 
    consentedVerbs : XApiVerbConsentedWithProvider[], 
    analyticTokenVerbs : AnalyticsTokenVerb []}[] = [];

  constructor(
    public dialogRef: MatDialogRef<VerbsModal>,
    @Inject(MAT_DIALOG_DATA) public data: { analysis : AnalyticsTokenConsent, providers : Provider[]},
    private _apiService: ApiService,
    private _toastrService: ToastrService
  ) {
      this.analysis = data.analysis
      this.providers = data.providers
      this._apiService = _apiService
      this._toastrService = _toastrService
      this.calculateProviderMapping()
  }

  ngOnInit()
  {
    this.dialogRef.updateSize('80%');
  }

  // allocate verbs to relevant providers
  calculateProviderMapping()
  {
      let mapping = [];
      for (let provider of this.providers)
      {
        let consentedVerbs = []
        let notConsentedVerbs = []
        let analyticTokenVerbs = []

        for(let verb of this.analysis.consentedVerbs)
          if(verb.providerId == provider.id)
          consentedVerbs.push(verb)


        for(let verb of this.analysis.notConsentedVerbs)
        if(verb.providerId == provider.id)
        notConsentedVerbs.push(verb)


        for(let verb of this.analysis.analyticTokenVerbs)
        if(verb.provider == provider.id)
          analyticTokenVerbs.push(verb)

        if(consentedVerbs.length || notConsentedVerbs.length || analyticTokenVerbs.length)  
          mapping.push({provider, notConsentedVerbs, consentedVerbs, analyticTokenVerbs})
      }

      this.providerMapping = mapping
  }

  ngOnChanges() {
    
  } 
 
  onNoClick(): void {
    this.dialogRef.close()
  }

  doAccept()
  {
    this._apiService.consentVerbsForAnalyticToken(this.analysis.id)
    .subscribe((analyticsTokens) => 
    {
      this._toastrService.success($localize`:@@toastMessageConsentSaved:The consent has been saved.`)

      this.dialogRef.close(true)
    })
  }

  hasNotConstentedElements()
  {
    return !!this.providerMapping.reduce( (acc, providerMap) => !!providerMap.notConsentedVerbs.length || acc, false)

  }

  hasConstentedElements()
  {
    return !!this.providerMapping.reduce( (acc, providerMap) => !!providerMap.consentedVerbs.length || acc, false)

  }
     
}