import { NgModule, APP_INITIALIZER } from '@angular/core'
import { BrowserModule } from '@angular/platform-browser'

import { AppRoutingModule } from './app-routing.module'
import { AppComponent } from './app.component'
import { FormsModule, ReactiveFormsModule } from '@angular/forms'
import { LayoutModule } from '@angular/cdk/layout'
import { HeaderComponent } from './navigation/header/header.component'
import {
    ConsentManagementComponent,
    PauseDataRecordingDialog
} from './consent-management/consent-management..component'
import { PageNotFoundComponent } from './page-not-found/page-not-found.component'
import { LoginPageComponent } from './login-page/login-page.component'
import { WizardDialog } from './consent-management/wizard/wizard.component'
import { FooterComponent } from './navigation/footer/footer.component'
import { LegalNoticeComponent } from './legal-notice/legal-notice.component'
import { HttpClientModule, HTTP_INTERCEPTORS } from '@angular/common/http'
import {
    PauseVerbObjectWarningDialog,
    PrivacySettingComponent
} from './consent-management/provider-settings/provider-setting.component'
import { ToastrModule } from 'ngx-toastr'
import { HttpRequestInterceptor } from './interceptors/http-request-interceptor'
import { JwtInterceptor } from './interceptors/jwt.interceptor'
import { HomeComponent } from './home/home.component'
import { appInitializer } from './app.initializer'
import { AuthService } from './services/auth.service'
import { LocalStorageService } from './services/localstorage.service'
import { ThemeService } from './services/theme.service'
import { ProviderComponent } from './consent-management/provider/provider.component'
import { UserProfilComponent } from './user-profil/user-profil.component'
import {
    ConfirmationDialog,
    DataDisclosureComponent
} from './data-disclosure/data-disclosure.component'
import { DataRemovalComponent } from './data-removal/data-removal.component'
import { MaterialDesignModule } from './material-design/material-design.module'
import { DeleteDialog } from './dialogs/delete-dialog/delete-dialog'
import { ApplicationTokensComponent } from './consent-management/application-tokens/application-tokens.component'
import { AnalyticsTokensComponent } from './consent-management/analytics-tokens/analytics-tokens.component'
import { SchemaChangeComponent } from './consent-management/schema-change/schema-change.component'
import { ObjectChangesComponent } from './consent-management/schema-change/object-changes/object-changes.component'
import { CreateTokenDialog } from './dialogs/create-token-dialog/create-token-dialog'
import { MAT_MOMENT_DATE_ADAPTER_OPTIONS } from '@angular/material-moment-adapter';
import { OrderByPipe } from './order-by.pipe';
import { MergeDataComponent } from './merge-data/merge-data.component'
import { AnalyticsEngineComponent } from './analytics-engine/analytics-engine.component'
import { AnalysisCard } from './analytics-engine/analysis-card/analysis-card.component'

@NgModule({
    declarations: [
        AppComponent,
        HeaderComponent,
        ConsentManagementComponent,
        PageNotFoundComponent,
        LoginPageComponent,
        WizardDialog,
        FooterComponent,
        LegalNoticeComponent,
        PrivacySettingComponent,
        HomeComponent,
        ProviderComponent,
        UserProfilComponent,
        DataDisclosureComponent,
        ConfirmationDialog,
        PauseDataRecordingDialog,
        PauseVerbObjectWarningDialog,
        DataRemovalComponent,
        DeleteDialog,
        ApplicationTokensComponent,
        AnalyticsTokensComponent,
        SchemaChangeComponent,
        ObjectChangesComponent,
        CreateTokenDialog,
        OrderByPipe,
        MergeDataComponent,
        AnalysisCard,
        AnalyticsEngineComponent
    ],
    imports: [
        BrowserModule,
        AppRoutingModule,
        MaterialDesignModule,
        LayoutModule,
        FormsModule,
        ReactiveFormsModule,
        HttpClientModule,
        ToastrModule.forRoot()
    ],
    providers: [
        {
            provide: APP_INITIALIZER,
            useFactory: appInitializer,
            multi: true,
            deps: [AuthService, LocalStorageService, ThemeService]
        },
        { provide: HTTP_INTERCEPTORS, useClass: HttpRequestInterceptor, multi: true },
        { provide: HTTP_INTERCEPTORS, useClass: JwtInterceptor, multi: true },
        { provide: MAT_MOMENT_DATE_ADAPTER_OPTIONS, useValue: { useUtc: true } },
    ],
    bootstrap: [AppComponent]
})
export class AppModule {}
