import { Component, OnInit, Inject } from '@angular/core'
import { ToastrService } from 'ngx-toastr'
import { DataDisclosure, DataDisclosureService } from '../data-disclosure.service'
import { environment } from '../../environments/environment'
import { MatDialog, MAT_DIALOG_DATA } from '@angular/material/dialog'

@Component({
    selector: 'app-data-disclosure',
    templateUrl: './data-disclosure.component.html',
    styleUrls: ['./data-disclosure.component.scss']
})
export class DataDisclosureComponent implements OnInit {
    exports: DataDisclosure[] = []
    isExportInProgress: boolean = true
    constructor(
        private _service: DataDisclosureService,
        private _toastrService: ToastrService,
        public dialog: MatDialog
    ) {}

    displayedColumns: string[] = ['id', 'status', 'created', 'expires', 'actions']

    ngOnInit(): void {
        this.getExports()
    }

    getExports() {
        this._service.list().subscribe((response) => {
            this.exports = response

            this.updateIsExportInProgress(response)
        })
    }

    updateIsExportInProgress(dataDisclosures: DataDisclosure[]) {
        for (const dataDisclosure of dataDisclosures) {
            if (!dataDisclosure.running && !dataDisclosure.status) {
                this.isExportInProgress = true
                return
            }
        }
        this.isExportInProgress = false
    }

    requestDataDisclosure(): void {
        this._service.create().subscribe((response) => {
            this._toastrService.success($localize`:@@successToastMessage:Request sent`)
            this.getExports()
        })
    }

    downloadZip(fileId: string): void {
        this._service.getFileSecret(fileId).subscribe((response) => {
            const url = `${environment.apiUrl}/api/v1/data-disclosure/files/${fileId}/${response.secret}`
            const link = document.createElement('a')
            link.setAttribute('target', '_blank')
            link.setAttribute('href', url)
            link.setAttribute('download', `example_config_polaris.json`)
            document.body.appendChild(link)
            link.click()
            link.remove()
        })
    }

    onDeleteClick(id: number): void {
        this.dialog.open(ConfirmationDialog, {
            width: '400px',
            data: {
                text: $localize`:@@deleteDataDisclosure:Delete data disclosure`,
                onConfirm: () => {
                    this._service.delete(id).subscribe((response) => {
                        this._toastrService.success(
                            $localize`:@@deleteDataDisclosureSucces:Data disclosure deleted`
                        ),
                            (this.exports = response)
                        this.updateIsExportInProgress(response)
                    })
                }
            }
        })
    }
}

export interface DialogData {
    text: string
    onConfirm: () => void
}

@Component({
    selector: 'confirmation-dialog',
    templateUrl: 'confirmation-dialog.html'
})
export class ConfirmationDialog {
    constructor(@Inject(MAT_DIALOG_DATA) public data: DialogData) {}
}
