import { Component, Inject } from '@angular/core'
import { FormBuilder, FormControl, Validators, FormGroup } from '@angular/forms'
import { MAT_DIALOG_DATA, MatDialogRef } from '@angular/material/dialog'

export interface DeleteDialogData {
    headerText: string
}

@Component({
    selector: 'delete-dialog',
    templateUrl: 'delete-dialog.html',
    styleUrls: ['./delete-dialog.scss']
})
export class DeleteDialog {
    confirmation = ''
    confirmationText = $localize`:@@deleteConfirmationText:delete`
    form: FormGroup
    deleteConfirmed = false

    constructor(
        public _dialogRef: MatDialogRef<DeleteDialog>,
        private _formBuilder: FormBuilder,
        @Inject(MAT_DIALOG_DATA) public data: DeleteDialogData
    ) {
        this.form = this._formBuilder.group({
            confirmation: ['', Validators.required]
        })
    }

    ngOnInit(): void {
        this.form.get('confirmation')?.valueChanges.subscribe((val) => {
            this.confirmation = val
            this.deleteConfirmed = val.localeCompare(this.confirmationText) === 0
        })
    }

    get confirmationControl(): FormControl {
        return this.form.get('confirmation') as FormControl
    }

    onSubmit(): void {
        if (this.deleteConfirmed) this._dialogRef.close(this.deleteConfirmed)
    }
}
