import { OverlayContainer } from '@angular/cdk/overlay'
import { Component, OnInit } from '@angular/core'
import { AuthService, Userinfo } from 'src/app/services/auth.service'
import { LanguageService } from 'src/app/services/language.service'
import { ThemeService } from 'src/app/services/theme.service'
import { environment } from 'src/environments/environment'

@Component({
    selector: 'app-header',
    templateUrl: './header.component.html',
    styleUrls: ['./header.component.scss']
})
export class HeaderComponent implements OnInit {
    loggedIn: Userinfo | null = null
    isDarkModeOn: boolean = false
    language: string = "de"

    constructor(
        private _authService: AuthService,
        private _theme: ThemeService,
        private _language: LanguageService,
        private _overlay: OverlayContainer
    ) {
        this._authService.loggedIn.subscribe((x) => (this.loggedIn = x))
        this._theme.darkModeSubject.subscribe((x) => (this.isDarkModeOn = x))
        this._language.language.subscribe((x) => (this.language = x))
    }

    ngOnInit(): void {
        if (this.isDarkModeOn) this._overlay.getContainerElement().classList.add('dark-mode')
    }

    logout(): void {
        this._authService.logout()
    }

    toggleTheme(): void {
        this._theme.toggleTheme()
        if (this.isDarkModeOn) this._overlay.getContainerElement().classList.add('dark-mode')
        else this._overlay.getContainerElement().classList.remove('dark-mode')
    }

    setLanguage(language: string) {
        this._language.setLanguage(language)
    }

    /**
     * Redirect user to SSO page.
     */
    handleSSOClick(): void {
        window.location.href = `${environment.apiUrl}/login`
    }
}
