from email.policy import default

from django.conf import settings
from django.db import models


class Provider(models.Model):
    definition_id = models.CharField(
        max_length=255
    )  # id gotten from uploaded definition
    name = models.CharField(max_length=200)
    nameI18n = models.JSONField(null=True)
    description = models.TextField()
    descriptionI18n = models.JSONField(null=True)
    updated = models.DateTimeField(auto_now=True)
    created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return "Provider: " + self.name


class ProviderSchema(models.Model):
    provider = models.ForeignKey(Provider, on_delete=models.CASCADE)
    superseded_by = models.ForeignKey("self", null=True, on_delete=models.SET_NULL)
    groups = models.JSONField()
    essential_verbs = models.JSONField()
    updated = models.DateTimeField(auto_now=True)
    created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return "Provider Schema."


class ProviderAuthorization(models.Model):
    provider = models.ForeignKey(Provider, on_delete=models.CASCADE)
    key = models.CharField(max_length=200)
    updated = models.DateTimeField(auto_now=True)
    created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.key


class AnalyticsToken(models.Model):
    name = models.CharField(max_length=64, db_index=True, unique=True)
    description = models.TextField(max_length=2500, null=True)
    creator = models.ForeignKey("users.CustomUser", on_delete=models.CASCADE)
    key = models.CharField(max_length=200)
    can_access = models.ManyToManyField(
        "self", related_name="result_access", symmetrical=False
    )
    expires = models.DateTimeField(null=True)
    created = models.DateTimeField(auto_now_add=True)
    image_path = models.CharField(max_length=250, null=True)
        
    def __str__(self):
        return "Analytics Token " + self.key

class AnalyticsTokenVerb(models.Model):
    verb = models.CharField(max_length=250)
    analytics_token = models.ForeignKey("providers.AnalyticsToken", on_delete=models.CASCADE)
    provider = models.ForeignKey(Provider, on_delete=models.CASCADE)
    created = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return "Analytics Token Verb: " + self.verb
